#include "lpc214x.h"


#define MAX_PINS 32
#define INPUT_BASE 8
#define OUTPUT_BASE 16
// pomocna struktura debounceru
static U16 buttonStates[MAX_PINS] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};

// udrzujeme si zde, jestli je dana ledka zapnuta nebo ne
  static U16 ledStates[MAX_PINS]    = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};

// A quick and dirty 'delay' routine
void delay(unsigned char ticks)
{
  unsigned long us = 1000*ticks;

  while (us--)
  {
    __asm volatile("nop");
  }
}


int pinRead(int pinNumber){
    return (GPIO0_IOPIN & (1 << pinNumber)) ? 1 : 0;
}

// musi byt volana pomerne casto, tak jednou za 5 - 10 ms
void buttonsDebounce()
{
    int i;
    // uint 16
    // uint 8
    U8 pinState;
    for (i = 0; i < MAX_PINS; i++){
	pinState = (U8)pinRead(i);
        buttonStates[i] = ((buttonStates[i] << 1) | pinState | 0xE000);
    }
}

int wasButtonPressed(int button_nr){
    int ret = 0;
    if (buttonStates[button_nr] == 0xF000) {
	ret = 1;
    }
    return ret;
}

void ledOn(int ledNum){
    ledStates[ledNum] = 1;
    GPIO0_IOCLR |= (1 << ledNum);
}

void ledOff(int ledNum){
  ledStates[ledNum] = 0;
  GPIO0_IOSET |= (1 << ledNum);
}

void ledToogle(int ledNum){  
  
  if (ledStates[ledNum] == 0){
      ledOn(ledNum);
  } else {
    ledOff(ledNum);
  }
}


void initPortRangeAsOutput(int fromPortNum, int toPortNum){
  int i;
  for (i = fromPortNum; i < toPortNum; i ++ ){
    GPIO0_IODIR |= (1 << i);
    ledOff(i);
  }
}

void initPortRangeAsInput(int fromPortNum, int toPortNum){
  int i;
  for (i = fromPortNum; i < toPortNum; i++ ){
    GPIO0_IODIR &= ~(1 << i);
  }
}

int main(void){
  int i;
  initPortRangeAsInput(INPUT_BASE, INPUT_BASE + 8);
  initPortRangeAsOutput(OUTPUT_BASE, OUTPUT_BASE + 8);  
  
  for (;;){
      
      buttonsDebounce();
      
      for (i = 0; i < 8; i++){
	if (1 == wasButtonPressed(INPUT_BASE + i)) {
	  ledToogle(OUTPUT_BASE + i);
	}
      }
      
      delay(1);
  }
}